
import cdsapi
from netCDF4 import Dataset
import numpy as np
import xarray as xr
import pandas as pd
import math
import glob
import netCDF4 as nc
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import cmaps
from matplotlib import rcParams
import cartopy.feature as cf
import matplotlib.patches as mpatchesv
import heapq
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from scipy import interpolate
from scipy.stats import pearsonr
import scipy.stats as stats
from scipy import integrate
import math
#from matplotlib import pyplot
import matplotlib.ticker as ticker

def T_test(data_fc,data_md,data3,data4):
    std1=np.zeros((data3.shape[1],data3.shape[2]))
    std2=np.zeros((data4.shape[1],data4.shape[2]))    
    for i in range(data3.shape[1]):
        for j in range(data3.shape[2]):
            std1[i,j]=np.std(data3[:,i,j])
            std2[i,j]=np.std(data4[:,i,j])
    
    z=np.zeros((data_fc.shape[0],data_fc.shape[1]))
    for i in range(data3.shape[1]):
        for j in range(data3.shape[2]):
            data_math1=std1[i,j]**2/(50)
            data_math2=std2[i,j]**2/(20)

            z[i,j]=(data_fc[i,j]-data_md[i,j])/np.sqrt(data_math1+data_math2)
            z1=abs(z)
    
    return z1

print('开始MD')

f1=nc.Dataset('cesm2.MD.ensmean.trend.SST.nc')
dts_slope=f1.variables['sst'][:,:].squeeze()

file1='cesm2.MD.ensmean.trend.budget.SST.nc'
f1=nc.Dataset(file1)
lon=f1.variables['lon'][:].squeeze()
lat=f1.variables['lat'][:].squeeze()
Tt1=f1.variables['sst'][:,:].squeeze()
Tt1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.SW.nc')
Tsw1=f1.variables['shortwave'][:,:].squeeze()
Tsw1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.LW.nc')
Tlw1=f1.variables['longwave'][:,:].squeeze()
Tlw1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.SH.nc')
Tsh1=f1.variables['sensible_heat'][:,:].squeeze()
Tsh1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.Ocn.nc')
Tocean1=f1.variables['ocean_dynamics_heat'][:,:].squeeze()
Tocean1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.LH_W.nc')
Flhw1=f1.variables['latent_heat_wind'][:,:].squeeze()
Flhw1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.LH_RH.nc')
Flhrh1=f1.variables['latent_heat_relative_humidity'][:,:].squeeze()
Flhrh1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.budget.LH_dT.nc')
Flhdt1=f1.variables['latent_heat_temperature_gradient'][:,:].squeeze()
Flhdt1_area=f1.variables['area'][:,:].squeeze()

f1=nc.Dataset('cesm2.MD.ensmean.trend.wind_MD.nc')
u_slope1=f1.variables['u'][:,:].squeeze()
v_slope1=f1.variables['v'][:,:].squeeze()

w_data=np.zeros((8))
c_data=np.zeros((8))
e_data=np.zeros((8))
data_fc_md=[Tt1,Tsw1,Tlw1,Tsh1,Tocean1,Flhw1,Flhrh1,Flhdt1]
for i in range(8):
    data=data_fc_md[i]
    data1=data[90:102,96:145]
    data1=data1.reshape(-1)
    data1=np.nanmean(data1,axis=0)
    w_data[i]=data1
    
    data2=data[90:102,144:185]
    data2=data2.reshape(-1)
    data2=np.nanmean(data2,axis=0)
    c_data[i]=data2
    
    data3=data[90:102,184:225]
    data3=data3.reshape(-1)
    data3=np.nanmean(data3,axis=0)
    e_data[i]=data3
    
bar_data=[w_data,c_data,e_data]

##############################################################Bar chart of FC-MD
fig = plt.figure(figsize=(16,11),dpi=700)
ax1 = plt.subplot2grid((3,2),(0,0),colspan=2,rowspan=1) # 
ax1.spines ['top'] .set_linewidth(1.1)
ax1.spines ['right'] .set_linewidth(1.1)
ax1.spines ['bottom'] .set_linewidth(1.1)
ax1.spines ['left'] .set_linewidth(1.1)
plt.sca(ax1)

x=[1,2,3,4,5,6,7,8]
tick_label=['${T^t_{S}}$','${T^t_{SW}}$','${T^t_{LW}}$','${T^t_{SH}}$','${T^t_{Ocn}}$','${T^t_{LH,W}}$','${T^t_{LH,RH}}$','${T^t_{LH,\Delta T}}$']

bar_width=0.16
index1 = np.arange(8) #
index2 = index1 + bar_width # 
index3 = index1 + 2*bar_width # 

plt.bar(index1,w_data,width=bar_width,align="center",color="#3B00DB",label="Western Pacific",edgecolor='black',\
        linewidth=1.1)
plt.bar(index2,c_data,width=bar_width,align="center",color="orange",label="Central Pacific",edgecolor='black',\
        linewidth=1.1)
plt.bar(index3,e_data,width=bar_width,align="center",color="#DC143C",label="Eastern Pacific",edgecolor='black',\
        linewidth=1.1)

plt.xticks(index2, tick_label,fontsize=23,fontdict={'fontname': 'Arial'})
plt.axhline(0, c='k', linestyle='-',linewidth=0.6,color='k',alpha=0.8)

my_y_ticks = np.arange(-0.24,0.121,0.06)
plt.yticks(my_y_ticks,
     ['-0.24','-0.18','-0.12','-0.06','0','0.06','0.12'],size=16,fontdict={'fontname': 'Arial'})
plt.xlabel("")
plt.ylabel("$^\circ$C/decade",fontsize=18,fontdict={'fontname': 'Arial'})
plt.legend(frameon=False,fontsize=14,loc="lower right",prop={'family': 'Arial'})

plt.grid(linestyle='--',alpha=0.2)   
plt.title('(a) Heat budget analysis for MD SST trend',size=22,fontdict={'fontname': 'Arial'},loc='left')#

##############################################################pattern
name=['(b) ${T^t_{S}}$','(c) ${T^t_{LH,W}}$','(d) ${T^t_{LW}}$','(e) ${T^t_{SW}}$']

Tt1=Tt1
Tsw1=Tsw1
Tlw1=Tlw1
Flhw1=Flhw1

data=[Tt1,Flhw1,Tlw1,Tsw1]
area11=[Tt1_area,Flhw1_area,Tlw1_area,Tsw1_area]

cmap_color=cmaps.BlueDarkRed18
label1111=['',f'{0.1} '+'m/s/decade','','']

levell=np.arange(-0.2,0.21,0.02)
range_number=[-0.2,-0.1,0,0.1,0.2]

alph1=[0,1,0,0]

for i in range(2):
    for j in range(2):
        proj=ccrs.PlateCarree(central_longitude=180)  ##
        ax1 = plt.subplot2grid((3,2),(1+i,0+j),colspan=1,rowspan=1,projection=proj)
        region = [120,290,-30,30]  ##

        ax1.spines ['top'] .set_linewidth(1.1)###
        ax1.spines ['right'] .set_linewidth(1.1)
        ax1.spines ['bottom'] .set_linewidth(1.1)
        ax1.spines ['left'] .set_linewidth(1.1)
        ax1.spines['geo'].set_linewidth(1.1)#
        plt.sca(ax1)

        land = cfeature.NaturalEarthFeature('physical', 'land', scale='50m', facecolor = cfeature.COLORS['land'])
        ax1.add_feature(land, facecolor = 'lightgray',zorder=1) ##这两步是把海岸线中的陆地填上颜色
        ax1.coastlines(linewidth = 1.2)  ##画出海岸线
        ax1.set_extent(region, crs = ccrs.PlateCarree())   ##选择区域
        ax1.set_xticks([120,150, 180, 210, 240,270],crs = ccrs.PlateCarree(),)
        ax1.set_yticks([-30, -15,0,15, 30],crs = ccrs.PlateCarree(),)
        lon_formatter = LongitudeFormatter(zero_direction_label=True)
        lat_formatter = LatitudeFormatter()
        ax1.xaxis.set_major_formatter(lon_formatter)
        ax1.yaxis.set_major_formatter(lat_formatter)

        ax1.xlabel_style = {'font.family':'Arial','size': 40, 'color': 'k'}  #####
        ax1.ylabel_style = {'font.family':'Arial','size': 40, 'color': 'k'}

        ax1.tick_params(direction = 'out',size=8,width=1.2)##
        font_title={'family':'Arial',
                    'weight':'normal',
                    'size':13}
        
        title = name[2*i+j]
        ax1.set_title(title,loc='left',fontdict={'fontname': 'Arial'},fontsize=24)

        title1 = rr[2*i+j]
        ax1.set_title(title1,loc='right',fontdict={'fontname': 'Arial'},fontsize=18)

        levels2=levell
        cs = ax1.contourf(lon, lat, data[2*i+j], levels=levels2,zorder=0,
                         transform=ccrs.PlateCarree(),
                         cmap = cmap_color,
                         extend = 'both') #
        sig2 = ax1.contourf(lon, lat, area11[2*i+j],colors='none',hatches=['..',None],transform=ccrs.PlateCarree(),zorder=0)
        
        step1=1
        step2=1

        Q = ax1.quiver(lon[::step2], lat[::step1],u_slope1[::step1,::step2], v_slope1[::step1,::step2],transform=ccrs.PlateCarree(),zorder=0,
         scale=0.4,units='inches', width=0.02, headwidth=2.4,color='k',alpha=alph1[2*i+j],regrid_shape=14)
        qk= ax1.quiverkey(Q,  0.62, 1.06, U=0.1, label=label1111[2*i+j],
                         angle=0,color='k', coordinates='axes',
                     labelpos='E',labelcolor='k',fc='k',labelsep=0.05,alpha=alph1[2*i+j],
                      fontproperties={'size': 19,'family':'Arial'})
        font_title={'family':'Arial',
                    'weight':'normal',
                    'size':18}

        config = {"font.family":'Arial', "font.size": 16, "mathtext.fontset":'stix'}
        rcParams.update(config)

cax = plt.axes([0.32, 0.05, 0.32, 0.02])  
cbar = fig.colorbar(cs,cax=cax,cmap = cmap_color, extend='both', orientation='horizontal')#
label1='$^\circ$C/decade'
cbar.set_label(label1,font=font_title)
cbar.ax.tick_params(direction = 'out',size=8,labelsize=16)#
tick_locator = ticker.MaxNLocator(nbins=5) 

cbar.locator = tick_locator
plt.tight_layout()
cbar.set_ticks(range_number)

cbar.update_ticks()
fig.subplots_adjust(left=0.099, bottom=0.099, right=0.869, top=0.948, wspace=0.165, hspace=0.227)
print('结束')
